package breakout;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.MouseInfo;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;
import java.util.logging.Level;

import javax.swing.ImageIcon;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

public class Brett extends JPanel implements MouseListener {
	private static final long serialVersionUID = -8427319957612167371L;
	private static final int LEVEL_END_DELAY = 30;
	private static final int GLUE_TIME = 2000;
	private int width = 400;
	private int height = 400;
	private Schlaeger schlaeger;
	private int leben = 0;
	private int score = 0;
	private int level = 0;
	private String levelName = "";
	private int delay = LEVEL_END_DELAY;
	private Random random = new Random();
	private int spielStatus = 1;
	private int gameOver = 0;
	private String soundId;
	private Image img = null;
	List<Ball> balls = new ArrayList<Ball>();
	List<Drop> drops = new ArrayList<Drop>();
	List<Block> blocks = new ArrayList<Block>();
	List<Ball> removeBalls = new ArrayList<Ball>();
	List<Drop> removeDrops = new ArrayList<Drop>();
	List<Block> removeBlocks = new ArrayList<Block>();

	List<GameChangeListener> gameChangeListeners = new ArrayList<GameChangeListener>();
	private Spiel spiel;
	private Sound sound = new Sound();

	public void addToScore(int i) {
		score += i;
		for (GameChangeListener g : gameChangeListeners) {
			g.scoreHasChanged(score);
		}
	}

	public void addBall(Ball go) {
		balls.add(go);
	}

	public void addDrop(Drop go) {
		drops.add(go);
	}

	public Brett() {
		setPreferredSize(new Dimension(width, height));
		setOpaque(true);
		this.addMouseListener(this);
	}

	public void paintComponent(Graphics g) {
		if( img != null ) g.drawImage(img, 0, 0, this.getWidth(),this.getHeight(),this);
		
		//super.paintComponent(g);
		Graphics2D g2 = (Graphics2D) g;

		for (Ball go : balls) {
			go.paintTo(g2);
		}
		for (Drop go : drops) {
			go.paintTo(g2);
		}
		for (Block b : blocks) {
			b.paintTo(g2);
		}
		if( schlaeger != null ) schlaeger.paintTo(g2);
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		Brett b = new Brett();
		b.setVisible(true);
	}

	/**
	 * Fill a level with a square of blocks
	 * 
	 * @param zmax Number of rows
	 */
	public synchronized void addBlocksSquare(int zmax) {
		int z = 1;
		while (z <= zmax + 2) {
			int s = 30;
			while (s < width - Block.BLOCK_WIDTH - 5) {
				int zp = 30 + z * (Block.BLOCK_HEIGHT + 4);
				Block b = null;
				if (z % 2 == 0 && s % 3 == 0) {
					b = new MultiBlock(s, zp);
				} else if (random.nextInt(100) > 90) {
					b = new IncPaddleBlock(s, zp);
				} else if (random.nextInt(100) > 90) {
					b = new DecPaddleBlock(s, zp);
				} else {
					b = new Block(s, zp);
				}
				b.setImage(spiel.getBmedia().getImage(b));
				blocks.add(b);

				s += Block.BLOCK_WIDTH + 4;
			}
			z++;
		}
		levelName = "squares";
		img = null;
	}

	/**
	 * Load the given level from file and build the blocks.
	 * 
	 * @param level number of the level to load
	 * @return true if the level has been found
	 */
	private boolean loadLevel(int level) {
		LevelLoader ll = new LevelLoader();
		breakout.Level levelDescription = ll.loadLevel(level);
		if( levelDescription == null ) return false;
		
		String[][] feld = levelDescription.getFeld();
		
		if( levelDescription.getImage() != null ) {
			img = new ImageIcon( levelDescription.getImage() ).getImage();
		}
		levelName = levelDescription.getName();
		
		int z = 1;
		for (String[] zeile : feld) {
			int zp = 30 + z * (Block.BLOCK_HEIGHT + 4);
			int s = 30;
			for (String blockName : zeile) {
				if (!blockName.equals("")) {
					Class c;
					try {
						c = Block.class.getClassLoader().loadClass("breakout."+blockName);
						Block b = (Block) c.newInstance(); 
						b.setPosition( s, zp );
						b.setImage(spiel.getBmedia().getImage(b));
						blocks.add(b);
					} catch ( Exception e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}
				s += Block.BLOCK_WIDTH + 4;
			}
			++z;
		}
		return true;
	}

	/**
	 * One move of the game. 
	 */
	public synchronized void move() {
		if (spielStatus == 1) {
			Point p = MouseInfo.getPointerInfo().getLocation();
			Point mp = spiel.getLocation();
			int dist = (int) (p.x - mp.x - schlaeger.x);
			schlaeger.setVx((int) (Math.signum(dist)) * Schlaeger.speed(dist));
			schlaeger.move();
			schlaeger.check(this);

			for (Drop d : drops) {
				d.move();
				d.check(this);
				if (d.hits(schlaeger)) {
					d.hit(this);
					removeDrops.add(d);
					addToScore(d.getScore());
				}
			}
			drops.removeAll(removeDrops);

			for (Ball ball : balls) {
				if (ball.hits(schlaeger) && ball.getVy() > 0) {
					ball.reflectV();
					ball.noZeroVX();
					ball.changeV(schlaeger.getVx() / 3, 0);
				}
				ball.move();
				ball.check(this);

				for (Block b : blocks) {
					if (ball.hits(b)) {
						b.hit(this);
						// ganz simple Reflexion,  nur vy = -vy 
						ball.reflectV();
					}
				}
			}

			if (!removeBalls.isEmpty()) {
				balls.removeAll(removeBalls);
			}
			if (balls.isEmpty()) {
				--leben;
				if (leben < 0) {
					spiel.getNotify(score);
					int n = JOptionPane.showConfirmDialog(this,
							"Game over \n Wollen Sie neu beginnen?",
							"Ja Nein Dialog", JOptionPane.YES_NO_OPTION,
							JOptionPane.QUESTION_MESSAGE);
					if (n == JOptionPane.YES_OPTION) {
						setDefaults();
					} else {
						System.exit(0);
					}
				}
				for (GameChangeListener g : gameChangeListeners) {
					g.liveHasChanged(leben);
				}
				glueBall(GLUE_TIME);
			}
			removeBalls.clear();

			for (Block b : removeBlocks) {
				blocks.remove(b);
			}
			removeBlocks.clear();
			if (blocks.isEmpty()) {
				--delay;
				if (delay <= 0 && drops.isEmpty()) {
					delay = LEVEL_END_DELAY;
					nextLevel();
				}
			}
		}
	}

	public synchronized void nextLevel() {
		++level;
		balls.clear();
		drops.clear();
		//addBlocksSquare(level);
		if( ! loadLevel(level) ) addBlocksSquare(level);
		for (GameChangeListener g : gameChangeListeners) {
			g.levelHasChanged(level, levelName);
		}
		if (schlaeger == null)
			createSchlaeger();
		glueBall(GLUE_TIME);
	}

	public void glueBall() {
		glueBall(0);
	}

	public void glueBall(int i) {
		Ball neu = new Ball((int) schlaeger.getX(), (int) schlaeger.getY()
				- schlaeger.getHeight(), 0, 0);
		neu.setGlueTimer(i);
		balls.add(neu);
		schlaeger.glue(neu);
	}

	public int getWidth() {
		return width;
	}

	public void setWidth(int width) {
		this.width = width;
	}

	public int getHeight() {
		return height;
	}

	public void setHeight(int height) {
		this.height = height;
	}

	public void createSchlaeger() {
		schlaeger = new Schlaeger(this);
	}

	public void removeBlock(Block block) {
		removeBlocks.add(block);
		soundId = "applaus";
		sound.spieleSound(soundId);
	}

	@Override
	public void mouseClicked(MouseEvent e) {
	}

	@Override
	public void mouseEntered(MouseEvent e) {
		spielStatus = 1;
	}

	@Override
	public void mouseExited(MouseEvent e) {
		spielStatus = 0;
		soundId = "piep";
		sound.spieleSound(soundId);
	}

	@Override
	public void mousePressed(MouseEvent e) {
	}

	@Override
	public void mouseReleased(MouseEvent e) {
		schlaeger.releaseKugel();
	}

	public void addChangeListener(GameChangeListener info) {
		gameChangeListeners.add(info);
		info.scoreHasChanged(score);
		info.liveHasChanged(leben);
	}

	public void setRemoveBall(Ball ball) {
		removeBalls.add(ball);
	}

	public void setRemoveDrop(Drop drop) {
		removeDrops.add(drop);
	}

	public void setSpiel(Spiel spiel) {
		this.spiel = spiel;
	}

	public void incPaddle() {
		schlaeger.addSize(10, 0);
	}

	public void decPaddle() {
		schlaeger.addSize(-10, 0);
		if (schlaeger.getWidth() < 15)
			schlaeger.setWidth(15);
	}

	public int gameOver() {
		soundId = "lachen";
		sound.spieleSound(soundId);
		return gameOver;
	}

	public void setDefaults() {
		leben = 0;
		score = 0;
		level = 1;
		for (GameChangeListener g : gameChangeListeners) {
			g.scoreHasChanged(score);
		}
		for (GameChangeListener g : gameChangeListeners) {
			g.liveHasChanged(leben);
		}
		for (GameChangeListener g : gameChangeListeners) {
			g.levelHasChanged(level, levelName);
		}
		balls.clear();
		drops.clear();
		blocks.clear();
		if( ! loadLevel(level) ) addBlocksSquare(level);
		schlaeger.setDefaultSchlaeger(this);
	}

	public void clear() {
		blocks.clear();
	}
}