package breakout;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;

import javax.swing.BoxLayout;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.Timer;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;

/**
 * Educational version of the BreakOut game.
 * 
 * @author Euler
 * @version 0.9 Mrz 2010
 */

public class Spiel extends JFrame implements ActionListener {
	public static final int DELTA_T = 15;
	static final String aboutText = "FH-Friedberg Breakout demo";
	static final String helptext = "No help available";
	public Highscore highscore = new Highscore();
	protected BMedia bmedia;
	Brett brett = new Brett();
	Info info = new Info();

	private String version = " Version 0.9 Mrz 2010";

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		(new Spiel()).start();
	}

	private void start() {
		bmedia = new BMedia( this );
		setDefaultCloseOperation( JFrame.EXIT_ON_CLOSE);
		setTitle("Break Out " + version );
		brett.setSpiel( this );
		getContentPane().setLayout(new BoxLayout(getContentPane(), BoxLayout.Y_AXIS));
		getContentPane().add( info );
		getContentPane().add( brett );
		pack();
		BMenu bMenu = new BMenu(this);
		setJMenuBar(bMenu.menuBar); 
		setVisible(true);		
		brett.addChangeListener( info );
		brett.nextLevel();
		repaint();
		Timer t = new Timer( DELTA_T, new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				doOnTick();
			}
		});
		t.start();
	}
	
	protected void doOnTick() {
		brett.move();
		repaint();
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		String cmd = e.getActionCommand();
		if (cmd.equals("F_Exit")) { 
			System.exit(0);
		} else if (cmd.equals("H_Version")) {
			JOptionPane.showMessageDialog(this, "Current version: "
					+ version );
		} else if (cmd.equals("H_About")) {
			JOptionPane.showMessageDialog(this, aboutText);
		} else if (cmd.equals("H_Help")) {
			JOptionPane.showMessageDialog(this, helptext); 
		} else if (cmd.equals("F_Highscore")) {
			JOptionPane.showMessageDialog(this, highscore.getHighscore()); 
		} else if (cmd.equals("F_New")) {
			brett.setDefaults(); 
		} else if (cmd.equals("C_NextLevel")) {
			brett.clear();
			brett.nextLevel();
		} else if (cmd.equals("F_ChangeColorset")) {
			ColorLoader.saveColorset( setColor() );
			BColor.refresh();
		}		
	}

	public String setColor(){
		String colorset = "0";
		try {
			colorset = ColorLoader.getColorset();
			if(colorset.equals("1")){
				colorset="0";
			}else{
				colorset="1";
			}
		} catch (JDOMException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return colorset;
	}
	
	public void getNotify(int score){
		try {
			try {
				highscore.writeHighscore(score);
			} catch (JDOMException e) {
				e.printStackTrace();
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * @return the bmedia
	 */
	public BMedia getBmedia() {
		return bmedia;
	}
}