package breakout;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.MouseInfo;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import javax.swing.JPanel;

public class Brett extends JPanel implements MouseListener {
	private static final long serialVersionUID = -8427319957612167371L;
	private static final int LEVEL_END_DELAY = 30;
	private static final int GLUE_TIME = 2000;
	private int width = 400;
	private int height = 400;
	private Schlaeger schlaeger;
	private int leben = 5;
	private int score = 0;
	private int level = 2;
	private int delay = LEVEL_END_DELAY;
	private Random random = new Random();

	List<Ball> balls = new ArrayList<Ball>();
	List<Drop> drops = new ArrayList<Drop>();
	List<Block> blocks = new ArrayList<Block>();
	List<Ball> removeBalls = new ArrayList<Ball>();
	List<Drop> removeDrops = new ArrayList<Drop>();
	List<Block> removeBlocks = new ArrayList<Block>();

	List<GameChangeListener> gameChangeListeners = new ArrayList<GameChangeListener>();
	private Spiel spiel;
	
	public void addToScore(int i) {
		score += i;
		for (GameChangeListener g : gameChangeListeners) {
			g.scoreHasChanged(score);
		}
	}

	public void addBall(Ball go) {
		balls.add(go);
	}

	public void addDrop(Drop go) {
		drops.add(go);
	}

	public Brett() {
		System.out.println("New Brett");
		setPreferredSize(new Dimension(width, height));
		this.addMouseListener(this);
	}

	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		Graphics2D g2 = (Graphics2D) g;
		for (Ball go : balls) {
			go.paintTo(g2);
		}
		for (Drop go : drops) {
			go.paintTo(g2);
		}
		for (Block b : blocks) {
			b.paintTo(g2);
		}
		schlaeger.paintTo(g2);
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		Brett b = new Brett();
		b.setVisible(true);
	}

	public synchronized void addBlocksSquare(int zmax) {
		int z = 1;
		while (z <= zmax) {
			int s = 30;
			while (s < width - Block.BLOCK_WIDTH - 5) {
				int zp = 30 + z * (Block.BLOCK_HEIGHT + 4);
				Block b = null;
				if (z % 2 == 0 && s % 3 == 0) {
					b = new MultiBlock(s, zp);
				} else	if ( random.nextInt(100) > 90 ) {
					b = new IncPaddleBlock(s, zp);
				} else	if ( random.nextInt(100) > 90 ) {
					b = new DecPaddleBlock(s, zp);
				} else {
					b = new Block(s, zp);
				}
				b.setImage( spiel.getBmedia().getImage( b));
				blocks.add(b);

				s += Block.BLOCK_WIDTH + 4;
			}
			z++;
		}

	}

	public synchronized void move() {
		
		Point p = MouseInfo.getPointerInfo().getLocation();
		Point mp = spiel.getLocation();
		int dist = (int)(p.x - mp.x - schlaeger.x);
		schlaeger.setVx((int) (Math.signum(dist)) * Schlaeger.speed(dist));
		schlaeger.move();
		schlaeger.check(this);

		for (Drop d : drops) {
			// System.out.println( d );
			d.move();
			d.check(this);
			if (d.hits(schlaeger)) {
				d.hit(this);
				removeDrops.add(d);
				addToScore(d.getScore());
			}
		}
		drops.removeAll(removeDrops);

		for (Ball ball : balls) {
			if (ball.hits(schlaeger) && ball.getVy() > 0) {
				ball.reflectV();
				ball.noZeroVX();
//				ball.setVx(ball.getVx() + schlaeger.getVx() / 3);
				ball.changeV(schlaeger.getVx() / 3, 0);
			}
			ball.move();
			ball.check(this);

			for (Block b : blocks) {
				if (ball.hits(b)) {
					b.hit(this);
					ball.reflectV();
				}
			}
		}

		if (!removeBalls.isEmpty()) {
			System.out.println("remove " + removeBalls.size() + " Blle");
			System.out.println("noch " + balls.size() + " Blle");
			balls.removeAll(removeBalls);
			System.out.println("noch " + balls.size() + " Blle");
		}
		if (balls.isEmpty()) {
			--leben;
			for (GameChangeListener g : gameChangeListeners) {
				g.liveHasChanged(leben);
			}
			System.out.println("noch " + leben + " Leben");
			glueBall( GLUE_TIME );
		}
		removeBalls.clear();

		for (Block b : removeBlocks) {
			blocks.remove(b);
		}
		removeBlocks.clear();
		if (blocks.isEmpty()) {
			--delay;
			if (delay <= 0 && drops.isEmpty()) {
				delay = LEVEL_END_DELAY;
				nextLevel();
			}
		}

	}

	public synchronized void nextLevel() {
		++level;
		System.out.println("level: " + level);
		balls.clear();
		drops.clear();
		addBlocksSquare(level);
		for (GameChangeListener g : gameChangeListeners) {
			g.levelHasChanged(level);
		}
		if (schlaeger == null)
			createSchlaeger();
		glueBall(GLUE_TIME);
	}

	public void glueBall() {
		glueBall(0);
	}

	public void glueBall(int i) {
		Ball neu = new Ball((int) schlaeger.getX(), (int) schlaeger.getY()
				- schlaeger.getHeight(), 0, 0);
		neu.setGlueTimer( i );
		balls.add(neu);
		schlaeger.glue(neu);
	}


	public int getWidth() {
		return width;
	}

	public void setWidth(int width) {
		this.width = width;
	}

	public int getHeight() {
		return height;
	}

	public void setHeight(int height) {
		this.height = height;
	}

	public void createSchlaeger() {
		schlaeger = new Schlaeger(this);
		// gameObject.add(schlaeger);

	}

	public void removeBlock(Block block) {
		removeBlocks.add(block);
	}

	@Override
	public void mouseClicked(MouseEvent e) {
		// TODO Auto-generated method stub

	}

	@Override
	public void mouseEntered(MouseEvent e) {
		// TODO Auto-generated method stub

	}

	@Override
	public void mouseExited(MouseEvent e) {
		// TODO Auto-generated method stub

	}

	@Override
	public void mousePressed(MouseEvent e) {
		// TODO Auto-generated method stub

	}

	@Override
	public void mouseReleased(MouseEvent e) {
		// System.out.println( "mouseReleased");
		schlaeger.releaseKugel();

	}

	public void addChangeListener(GameChangeListener info) {
		gameChangeListeners.add(info);
		info.scoreHasChanged(score);
		info.liveHasChanged(leben);
	}

	public void setRemoveBall(Ball ball) {
		removeBalls.add(ball);
	}

	public void setRemoveDrop(Drop drop) {
		removeDrops.add(drop);
	}

	public void setSpiel(Spiel spiel) {
		this.spiel = spiel;
		
	}

	public void incPaddle() {
		schlaeger.addSize( 10, 0 );
	}

	public void decPaddle() {
		schlaeger.addSize( -10, 0 );
		if( schlaeger.getWidth() < 15 ) schlaeger.setWidth(15);
	}

}
